"""
.. codeauthor:: Nathan Hertz <nhertz@nrao.edu>

File containing definitions for the other half of the capability side of the Workspaces REST API,
concerning capability requests
"""

from pyramid.httpexceptions import HTTPBadRequest, HTTPNotFound, HTTPPreconditionFailed
from pyramid.request import Request
from pyramid.response import Response
from pyramid.view import view_config


@view_config(route_name="view_capability_request", renderer="json")
def view_capability_request(request: Request) -> Response:
    """
    Pyramid view that accepts a request to view a capability request and responds with the request's info, if it exists
    URL: capability/{capability_name}/request/{request_id}

    :param request: GET request
    :return: 200 OK response with JSON-formatted capability request info if found
        or 404 response (HTTPNotFound)
    """
    capability_request = request.capability_info.lookup_capability_request(
        request.matchdict["request_id"]
    )
    if capability_request:
        return Response(json_body=capability_request.__json__())
    else:
        not_found_msg = (
            f"Capability request for capability {request.matchdict['capability_name']}",
            f"with ID {request.matchdict['request_id']} not found.",
        )
        return HTTPNotFound(detail=not_found_msg)


@view_config(route_name="create_capability_request", renderer="json")
def create_capability_request(request: Request) -> Response:
    """
    Pyramid view that accepts a request to create a capability request
    URL: capability/{capability_name}/request/create

    :param request: POST request, expecting JSON parameter "parameters"
    :return: 200 OK response with JSON-formatted info of newly created capability request
        or 400 response (HTTPBadRequest) if expected parameters not given
        or 412 response (HTTPPreconditionFailed) if capability with given name does not exist and thus cannot be
        requested
    """
    expected_params = ["parameters"]
    capability_name = request.matchdict["capability_name"]
    params = request.json_body

    if not all([expected in params for expected in expected_params]):
        # JSON params do not contain all expected params
        params_not_given_msg = (
            f"Expected JSON parameters {expected_params}. Received only {params}."
        )
        return HTTPBadRequest(detail=params_not_given_msg)
    elif not request.capability_info.lookup_capability(capability_name):
        # Capability with given name does not exist; can't create a request for it
        does_not_exist_msg = (
            f"Capability {capability_name} does not exist. Cannot create request.",
        )
        return HTTPPreconditionFailed(detail=does_not_exist_msg)
    else:
        # TODO: Implement future products
        new_capability_request = request.capability_service.create_request(
            capability_name, parameters=params["parameters"], products=None
        )
        return Response(json_body=new_capability_request.__json__())


@view_config(route_name="edit_capability_request", renderer="json")
def edit_capability_request(request: Request) -> Response:
    """
    Pyramid view that accepts a request to edit a capability request
    URL: capability/{capability_name}/request/{request_id}/edit

    :param request: POST request
    :return Bad Request response, since it's not yet implemented

    TODO: Implement once CapabilityVersions are supported
    """
    return HTTPBadRequest(detail="Editing capability requests is not yet implemented.")


@view_config(route_name="submit_capability_request", renderer="json")
def submit_capability_request(request: Request) -> Response:
    """
    Pyramid view that accepts a request to submit a capability request
    URL: capability/{capability_name}/request/{request_id}/submit

    :param request: POST request
    :return: 200 OK response
        or 412 response (HTTPPreconditionFailed) if capability request with given ID does not exist and thus cannot be
        submitted
    """
    request_id = request.matchdict["request_id"]
    capability_request = request.capability_info.lookup_capability_request(request_id)

    if not capability_request:
        # Capability request not found
        does_not_exist_msg = (
            f"Capability request with ID {request_id} does not exist. Cannot submit."
        )
        return HTTPPreconditionFailed(detail=does_not_exist_msg)
    else:
        execution = request.capability_service.run_capability(capability_request)
        return Response(json_body=execution.__json__())


@view_config(route_name="delete_capability_request", renderer="json")
def delete_capability_request(request: Request) -> Response:
    """
    Pyramid view that accepts a request to delete a capability request from the face of the archive
    URL: capability/{capability_name}/request/{request_id}

    :param request: DELETE request
    :return: 200 OK response
        or 412 response (HTTPPreconditionFailed) if capability request with given ID does not exist and thus cannot be
        deleted
        or 400 response (HTTPBadRequest) if capability request with given ID cannot be deleted
    """
    capability_name = request.matchdict["capability_name"]
    request_id = request.matchdict["request_id"]
    capability_request = request.capability_info.lookup_capability_request(request_id)

    if not capability_request:
        # Capability request not found
        does_not_exist_msg = f"Capability request for {capability_name} with ID {request_id} does not exist. Cannot delete."
        return HTTPPreconditionFailed(detail=does_not_exist_msg)
    else:
        rows_deleted = request.capability_info.delete_capability_request(request_id)
        if rows_deleted == -1:
            bad_request_msg = (
                f"Capability request for {capability_name} with ID {request_id} cannot be deleted;",
                f"it has existing executions.",
            )
            return HTTPBadRequest(detail=bad_request_msg)
        return Response(
            body=f"Capability request for {capability_name} with ID {request_id} successfully deleted."
        )
